from setuptools import setup, find_packages

setup(
    name="diamond-hpc",
    version="0.0.3",
    author="Haotian XIE, Gengcong YANG",
    author_email="hotinexie@gmail.com",
    description="Diamond is a Python package for running tasks on HPC.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Diamond-Proj/Diamond",
    packages=find_packages(),
    package_data={
        "diamond": ["diamond_client/templates/*"]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "click>=8.1.3",
        "globus_compute_sdk>=2.20.0",
        "Jinja2>=3.1.4",
        "Requests>=2.32.2",
    ],
    entry_points={
        'console_scripts': [
            'diamond-hpc=diamond.wrapper.wrapper:cli',
        ],
    },
    python_requires='>=3.8',
)



import os
import pkg_resources
import platform
import tempfile

from diamond.container.container import ImageRegistry
from globus_sdk import NativeAppAuthClient
from globus_compute_sdk.sdk.login_manager.globus_auth  import internal_auth_client
from globus_compute_sdk.sdk.login_manager.manager import LoginManager


class DiamondClient:

    def __init__(self):
        pass

    def get_auth_client(self):
        auth_client = internal_auth_client()
        login_manager = LoginManager()
        scopes = [
            s for _rs_name, rs_scopes in login_manager.login_requirements for s in rs_scopes
        ]
        auth_client.oauth2_start_flow(
            redirect_uri=auth_client.base_url + "v2/web/auth-code",
            refresh_tokens=True,
            requested_scopes=scopes,
            prefill_named_grant=platform.node(),
        )
        return auth_client

    def get_oauth2_url(self, auth_client: NativeAppAuthClient):
        query_params = {"prompt": "login"}
        oauth2_url = auth_client.oauth2_get_authorize_url(query_params=query_params)
        return oauth2_url
    
    def oauth2_login_by_token(self, auth_client: NativeAppAuthClient, auth_code: str):
        auth_client.oauth2_exchange_code_for_tokens(auth_code)

    def generate_temp_file(self):
        return tempfile.mktemp()

    def generate_task_script(self, cmd: str, log_path: str, endpoint_id: str, container_id: str):
        task_template_filepath = pkg_resources.resource_filename(
            'diamond','diamond_client/templates/task_template')
        tmp_script_path = self.generate_temp_file()
        with open(task_template_filepath, 'r') as f:
            task_template = f.read()
            task_script = (
                task_template
                .replace("dummy_command", cmd)
                .replace("dummy_log_path", log_path)
                .replace("dummmy_endpoint_id", endpoint_id)
                .replace("dummy_container_id", container_id)
            )
            with open(tmp_script_path, 'w') as f:
                f.write(task_script)
        print(tmp_script_path)
        return tmp_script_path
    
    def run_task(self, cmd: str, log_path: str, endpoint_id: str, container_id: str):
        task_script_path = self.generate_task_script(cmd, log_path, endpoint_id, container_id)
        return os.system(f'python3 {task_script_path}')
    
    def register_container(
            self,
            endpoint_id: str,
            work_path: str,
            image_file_name: str,
            base_image: str):
        site_config = {
            "endpoint": endpoint_id,
            "work_path": work_path,
        }
        ir = ImageRegistry(site_config)
        container_id = ir.build(base_image, None, image_file_name)
        return container_id



import click

from diamond.diamond_client.diamond_client import DiamondClient


def execute_task(cmd: str, log_path: str, endpoint_id: str, container_id: str):
    diamond = DiamondClient()
    return diamond.run_task(cmd, log_path, endpoint_id, container_id)


@click.group()
def cli():
    pass


@cli.command()
@click.option('--cmd', required=True, help='Command to run task')
@click.option('--log_path', required=True, help='Log file path in the supercomputer')
@click.option('--endpoint_id', required=True, help='Endpoint ID')
@click.option('--container_id', required=True, help='Container ID')
def run_task(cmd: str, log_path: str, endpoint_id: str, container_id: str):
    click.echo(execute_task(cmd, log_path, endpoint_id, container_id))


@cli.command()
@click.option('--cmd_script', required=True, help='Task command script file path')
@click.option('--log_path', required=True, help='Log file path in the supercomputer')
@click.option('--endpoint_id', required=True, help='Endpoint ID')
@click.option('--container_id', required=True, help='Container ID')
def run_task_from_script(cmd_script: str, log_path: str, endpoint_id: str, container_id: str):
    with open(cmd_script, 'r') as f:
        cmd = f.read()
    click.echo(execute_task(cmd, log_path, endpoint_id, container_id))


@cli.command()
@click.option('--endpoint_id', required=True, help='Endpoint ID')
@click.option('--work_path', required=True, help='Work path')
@click.option('--image_file_name', required=True, help='Image file name')
@click.option('--base_image', required=True, help='Base image name in docker hub')
def register_container(endpoint_id: str,work_path: str,image_file_name: str,base_image: str):
    diamond = DiamondClient()
    container_id = diamond.register_container(endpoint_id, work_path, image_file_name, base_image)
    click.echo(f"The container id is {container_id}")


@cli.command()
def get_oauth2_url_and_login():
    diamond = DiamondClient()
    auth_client = diamond.get_auth_client()
    click.echo(diamond.get_oauth2_url(auth_client))
    auth_code = input("Please input the auth code: ")
    click.echo(diamond.oauth2_login_by_token(auth_client, auth_code))


if __name__ == '__main__':
    cli()
